/* src/add_info.c - Ajout d'informations en mmoire
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "main.h"

CVSINFO("$Id: add_info.c,v 1.43 2003/07/30 17:53:49 romexzf Exp $");

extern struct badchans *badchead;
extern aKill *killhead;
extern struct badnicks *badnhead;
extern struct ignore *ignorehead;

aNick *add_nickinfo(const char *nick, const char *user, const char *host, const char *base64, const char *num, struct Link *server, const char *name, time_t ttmco)
{
	aNick *new = calloc(1, sizeof(aNick));

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aNick %s !\n", nick);
		return NULL;
	}
	if(!(nick && user && host && base64 && num))
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_nickinfo\n");
		free(new);
		return NULL;
	}

	new->next = NULL;
	strncpy(new->nick, nick, NICKLEN);
	strncpy(new->ident, user, USERLEN);
	strncpy(new->host, host, HOSTLEN);
	strncpy(new->base64, base64, 7);
	strcpy(new->numeric, num);
	new->serveur = server;
	strncpy(new->name, name, REALEN);
	new->ttmco = ttmco;
	new->floodtime = time(NULL);
	new->floodcount = 0;
	new->last = NULL;
	if(!nickhead) nickhead = new;
	else
	{
		nickhead->last = new;
		new->next = nickhead;
		nickhead = new;
	}
	return new;
}

#ifdef USE_MEMOSERV
void add_memo(anUser *nick, const char *de, time_t date, const char *message, int read)
{
	aMemo *new = calloc(1, sizeof(aMemo)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aMemo %s !\n", nick->nick);
		return;
	}
	if(!(nick && de && message))
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS: add_memo\n");
		free(new);
		return;
	}

	new->next = NULL;
	strncpy(new->de, de, NICKLEN);
	new->date = date;
	new->read = read;
	strncpy(new->message, message, MEMOLEN);
	if(!nick->memohead) nick->memohead = new;
	else
	{
		for(current=nick->memohead;current->next;current=current->next);
		if(!current->next) current->next = new;
		else free(new);
	}
}
#endif

struct badchans *add_abadchan(const char *chaine, const char *raison)
{
	struct badchans *new = calloc(1, sizeof(struct badchans)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un badchans (%s) !\n", chaine);
		return NULL;
	}
	if(!(chaine && raison))
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_abadchan\n");
		free(new);
		return NULL;
	}
	new->next = NULL;
	strncpy(new->chaine, chaine, 99);
	strncpy(new->raison, raison, 149);
	if(!badchead) badchead = new;
	else
	{
		for(current=badchead;current->next && match(current->chaine, new->chaine);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
	return new;
}

aChan *add_chan(const char *chan, const char *topic, const char *description, const char *defmodes,
	const char *deftopic, const char *welcome, const char *motd, const char *chanurl)
{
	aChan *new = calloc(1, sizeof(aChan)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aChan %s !\n", chan);
		return NULL;
	}
	if(!chan)
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_chan\n");
		free(new);
		return NULL;
	}
	new->next = NULL;
	strncpy(new->chan, chan, REGCHANLEN);
	strncpy(new->topic, topic, TOPICLEN);
	strcpy(new->defmodes, defmodes);
	strncpy(new->deftopic, deftopic, TOPICLEN);
	strncpy(new->description, description, DESCRIPTIONLEN);
	strncpy(new->welcome, welcome, TOPICLEN);
	new->suspend = NULL;
	new->banlevel = DEFAUT_BANLEVEL;
	new->cml = DEFAUT_CMODELEVEL;
	new->bantype = 1;
	new->users = 1;
	if(motd && *motd) str_dup(&new->motd, motd);
	else new->motd = NULL;
	strncpy(new->url, chanurl, 100);
	new->banhead = NULL;

	if(!chanhead) chanhead = new;
	else
	{
		for(current=chanhead;current->next && strcasecmp(current->chan, chan);current=current->next);
		if(!current->next) current->next=new;
		else
		{
			free(new);
			new = NULL;
		}
	}
	return new;
}

anAccess *add_access(anUser *user, const char *chan, int level, int flag, time_t lastseen)
{
	anAccess *new = calloc(1, sizeof(anAccess)), *current;
	aChan *c;
	aLink *lp;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour anAccess %s sur %s!\n", user->nick, chan);
		return NULL;
	}
	if(!(user && chan))
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_access\n");
		free(new);
		return NULL;
	}
	new->next = NULL;
	new->level = level;
	new->flag = flag;
	new->user = user;
	new->info = NULL;
	strncpy(new->chan, chan, REGCHANLEN);
	if((c = getchaninfo(chan))) new->c = c;
	else
	{
		free(new);
		return NULL;
	}

	if(lastseen) new->lastseen = lastseen;
	else new->lastseen = user->lastseend;

	if(!user->accesshead) user->accesshead = new;
	else
	{
		for(current=user->accesshead;current->next && strcasecmp(current->chan,chan);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}

	lp = calloc(1, sizeof(aLink));
	lp->value.a = new;
	lp->next = c->access;
	c->access = lp;
	return new;
}

void add_killinfo(aNick *nick, time_t timestamp, int why, int setkill)
{
	aKill *new = calloc(1, sizeof(aKill)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour killinfo %s !\n", nick->nick);
		return;
	}
	if(!nick)
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_killinfo\n");
		free(new);
		return;
	}
	new->next = NULL;
	new->nick = nick;
	strncpy(new->against, nick->nick, NICKLEN);
	new->timestamp = timestamp;
	new->why = why;
	new->setkill = setkill;
	if(!killhead) killhead = new;
	else
	{
		for(current=killhead;current->next;current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

void add_abadnick(const char *chaine, int flag)
{
	struct badnicks *new = calloc(1, sizeof(struct badnicks)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un badnick %s !\n", chaine);
		return;
	}
	if(!chaine)
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_abadnick\n");
		free(new);
		return;
	}
	new->next = NULL;
	strncpy(new->chaine, chaine, NICKLEN);
	new->flag = flag;
	if(!badnhead) badnhead = new;
	else
	{
		for(current = badnhead;current->next && match(current->chaine, new->chaine);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

anUser *add_regnick(const char *user, const char *pass, time_t lastseend, int level, int flag, const char *mail, int vote)
{
	anUser *new;

	if(!(new = calloc(1,sizeof(struct userinfo))))
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour anUser %s !\n", user);
		return NULL;
	}
	if(!(user && pass && mail && level))
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_regnick\n");
		free(new);
		return NULL;
	}
	new->next = NULL;
	strncpy(new->nick, user, NICKLEN);
	strcpy(new->passwd, pass);
	new->lastseend = lastseend;
	new->level = level;
	strncpy(new->mail, mail, 100);
#ifdef HAVE_VOTE
	new->vote = vote;
#endif
	new->flag = flag;
	new->last = NULL;
	new->suspend = NULL;
	new->lastlogin = NULL;
	new->cantregchan = -1;
	if(!userhead) userhead = new;
	else
	{
		userhead->last = new;
		new->next = userhead;
		userhead = new;
	}
	return new;
}

void add_join(aNick *nick, char *chan, int is_op, int is_voice, time_t timestamp, int is_burst, aChan *chaninfo)
{

	aJoin *new = calloc(1, sizeof(aJoin)), *join;
	anAccess *access;
	aLink *lp, *l;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aJoin %s sur %s !\n", nick->nick, chan);
		return;
	}
	if(!(nick && chan))
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_join\n");
		free(new);
		return;
	}
	new->next = NULL;
	strncpy(new->channel, chan, CHANLEN);
	new->is_op = is_op;
	new->is_voice = is_voice;
	new->timestmp = timestamp;
	new->nick = nick;
	new->chan = NULL;
	if(!nick->joinhead) nick->joinhead = new;
	else
	{
		for(join = nick->joinhead;join->next && strcasecmp(join->channel, chan);join=join->next);
		if(!join->next && strcasecmp(join->channel, chan)) join->next=new;
		else /* le mec est dja sur le chan ? */
		{
			free(new);
			new = NULL;
		}
	}

	if((!is_burst && !(chaninfo = getchaninfo(chan))) || !chaninfo || IsSuspend(chaninfo)) return;

	new->chan = chaninfo;
	chaninfo->users++;

	if(new) /* on ajoute le link vers chanmembers uniquement si le salon n'est PAS suspendu */
	{		/* pb vu lors du burst d'un chan non suspendu */
		lp = calloc(1, sizeof(aLink));
		lp->value.j = new;
		if(!chaninfo->members) chaninfo->members = lp;
		else
		{
			for(l = chaninfo->members;l->next;l=l->next); /* on cherche le dernier */
			l->next = lp;
		}
	}

	if(is_burst) return;
	else
	{ /*ban check (mis ici pr eviter 2 loops)*/
		if(!(nick->flag & N_OPER) && (!nick->user || !IsAnAdmin(nick->user)))
		{
			aBan *ban;
			if((ban = is_ban(nick, chaninfo)))
			{
				csmode(chaninfo, "+b %s", ban->mask);
				cskick(chan, nick->numeric, "Enforce: %s", ban->raison);
				return;
			}
		}

		if(CSetWelcome(chaninfo) && chaninfo->welcome[0] != '\0') /*si le welcome est activ ET que le welcome existe*/
		   csntc(nick, "Bienvenue sur \2%s\2 - %s", chaninfo->chan, chaninfo->welcome);
	}
	/*On regarde l'access ici pour infoline, autoop et autovoice*/
	if(!nick->user || IsSuspend(nick->user)) return; /*ne va pas plus loin si l'user n'est pas auth ou qu'il est suspend (peut etre inutile)*/

	if(!(access = GetAccessIbyUserI(nick->user,chan)) || ASuspend(access)) return;/*ne va pas plus loin si l'user n'a pas d'access ou qu'il est suspendu*/
	access->lastseen = 1;

	if(AOp(access) || AVoice(access))
	{
		new->is_voice = AVoice(access);
		new->is_op = CNoOps(chaninfo) ? 0 : AOp(access);

		csmode(chaninfo, "+%s%s %s %s", AOp(access) ? (!CNoOps(chaninfo) ? "o" : ""):"",
			AVoice(access) ? "v" : "", AOp(access) ? (!CNoOps(chaninfo) ? nick->numeric : "") : "" , AVoice(access) ? nick->numeric : "");
	}
	if(access->info) putserv("%s " TOKEN_PRIVMSG " %s :[\2%s\2] %s\n", cs.num, chan, nick->user->nick, access->info);

	return;
}

void add_ban(aChan *cchan, const char *mask, const char *raison, const char *de, time_t debut, time_t fin, int level)
{
	aBan *new = calloc(1, sizeof(aBan)), *current;
	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aBan %s sur %s !\n", mask, cchan->chan);
		return;
	}
	if(!(cchan && mask))
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_ban\n");
		free(new);
		return;
	}
	strncpy(new->mask, mask, 99);
	str_dup(&new->raison, raison);
	strncpy(new->de, de, NICKLEN);
	new->debut = debut;
	new->fin = fin;
	new->level = level;
	new->next = NULL;
	new->last = NULL;
	if(!cchan->banhead) cchan->banhead = new;
	else
	{
		int i = 0;
		for(current = cchan->banhead;(i = match(current->mask, mask)) && current->next;current=current->next);
		if(i && !current->next)
		{
			current->next = new;
			new->last = current;
		}
		else free(new);
	}
	return;
}

void add_ignore(const char *host, time_t expiration)
{
	struct ignore *new = calloc(1, sizeof(struct ignore)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un ignore %s !\n", host);
		return;
	}
	if(!host)
	{
		putlog(LOG_ERREURS, "ERREUR D'ARGUMENTS : add_ignore\n");
		free(new);
		return;
	}
	new->next = NULL;
	strncpy(new->host, host, HOSTLEN);
	new->exp = expiration;
	if(!ignorehead) ignorehead = new;
	else
	{
		for(current = ignorehead;current->next;current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
	return;
}

