/* src/moduser.c - Modification des paramtres d'un accs
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "main.h"

CVSINFO("$Id: moduser.c,v 1.19 2003/07/29 09:19:54 romexzf Exp $");

/*
 * generic_moduser
 *
 * parv[1] = #salon
 * parv[2] = option
 * parv[3] = nick
 * parv[4] = argument
 */
int generic_moduser(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *user;
	anAccess *access, *acces;
	const char *c = parv[1];

	if(!(user = ParseNickOrUser(nick, parv[3]))) return 0;

	if(!(access = GetAccessIbyUserI(user, c)))
		return csntc(nick, "\2%s\2 n'a pas d'access sur \2%s\2", user->nick, c);

	if(!strcasecmp(parv[2], "protect"))
	{
		ToggleFlag(&access->flag, A_PROTECT);
		csntc(nick, "La protection a t %sactive pour \2%s\2 sur \2%s\2.", AProtect(access) ? "" : "des", user->nick, c);
	}

	else if(!strcasecmp(parv[2], "autoop"))
	{
		ToggleFlag(&access->flag, A_OP);
		csntc(nick, "L'AutoOp a t %sactiv pour \2%s\2 sur \2%s\2.", AOp(access) ? "" : "des", user->nick, c);
	}

	else if(!strcasecmp(parv[2], "autovoice"))
	{
		if(nick->user != user && (access->level < 50 && !IsAnAdmin(nick->user)))
			return csntc(nick, "Vous n'avez pas access  la commande voice");

		ToggleFlag(&access->flag, A_VOICE);
		csntc(nick, "L'AutoVoice a t %sactiv pour \2%s\2 sur \2%s\2.", AVoice(access) ? "" : "des", user->nick, c);
	}

	else if(!strcasecmp(parv[2], "suspend"))
	{
		if(!IsAnAdmin(nick->user) && (!(acces = GetAccessIbyUserI(nick->user, c)) || access->level >= acces->level))
			return csntc(nick, "Vous ne pouvez pas suspendre un niveau suprieur ou gal au votre.");

		ToggleFlag(&access->flag, A_SUSPEND);
		csntc(nick, "\2%s\2 a t %ssuspendu sur \2%s\2.", user->nick, ASuspend(access) ? "" : "des", c);
	}

	else if(!strcasecmp(parv[2], "level"))
	{
		int l;
		if(parc < 4 || !is_num(parv[4]))
			return csntc(nick, "Veuillez prciser un level valide.");

		if(access->level == OWNERLEVEL)
			return csntc(nick, "L'owner ne peut pas changer de niveau.");

		if((l = atoi(parv[4])) < 1 || l > OWNERLEVEL - 1)
			return csntc(nick, "Le niveau doit tre compris entre 1 et %d.", OWNERLEVEL - 1);

		if(!IsAnAdmin(nick->user))/*si c un admin pas de verif pr dep/dest*/
		{
			acces = GetAccessIbyUserI(nick->user, c); /*on recup les infos de l'executant*/
			if(l >= acces->level)
				return csntc(nick, "Vous ne pouvez donner un niveau suprieur ou gal au votre.");

			if(acces->level <= access->level)
				return csntc(nick, "Vous ne pouvez changer le niveau d'un user de niveau suprieur ou gal au votre.");

		}	/*il a pass les tests, on file a l'execution*/
		access->level = l;
		csntc(nick, "Le niveau de \2%s\2 a t chang en \002%d\2 sur \2%s", user->nick, l, access->chan);
	}

	else if(!strcasecmp(parv[2], "info"))
	{
		if(parc < 4)
			return csntc(nick, "Veuillez prciser une infoline ou le mot cl 'none' pour la supprimer");

		if(!strcasecmp(parv[4], "none"))
		{
			if(access->info)
			{
				free(access->info);
				access->info = NULL;
			}
			csntc(nick, "L'infoline de \2%s\2 sur \2%s\2 a t supprime.", user->nick, access->chan);
		}
		else
		{
			str_dup(&access->info, parv2msg(parc, parv, 4, 199));
			csntc(nick, "Infoline de \2%s\2 sur \2%s\2 change en: %s", user->nick, access->chan, access->info);
		}
	}

	else if(*parv[2] == '+' || *parv[2] == '-')
	{
		int w = 1;
		/* change flag +flag-flag*/
		while(*parv[2])
		{
			switch(*parv[2])
			{
				case '+': w = 1; break;
				case '-': w = 0; break;
				case 'o':
						if(w) access->flag |= A_OP;
						else access->flag &= ~A_OP;
						break;
				case 'p':
						if(w) access->flag |= A_PROTECT;
						else access->flag &= ~A_PROTECT;
						break;
				case 'v':
						if(w) access->flag |= A_VOICE;
						else access->flag &= ~A_VOICE;
						break;
				case 's':
						if(user == nick->user) csntc(nick, "Vous ne pouvez vous suspendre.");
						else
						{
							anAccess *a2 = GetAccessIbyUserI(nick->user, c);
							if(a2->level <= access->level)
							{
								csntc(nick, "Vous ne pouvez suspendre un Accs de niveau suprieur ou gal au votre.");
								break;
							}
							if(w) access->flag |= A_SUSPEND;
							else access->flag &= ~A_SUSPEND;
						}
						break;
			}
			parv[2]++;
		}
		csntc(nick, "Les Options de %s sur %s sont dsormais: %s", user->nick, c, GetAccessOptions(access));
	}
	else return csntc(nick, "%s n'est pas une option de moduser valide.", parv[2]);
	write_anuser(user);
	return 1;
}


int moduser_protect(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
   parv[2] = "PROTECT";
   if(parc == 2) parv[4] = parv[3];
   parv[3] = nick->user->nick;

   return generic_moduser(nick, chaninfo, parc == 2 ? 4 : 3, parv);
}

int moduser_autoop(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
   parv[2] = "AUTOOP";
   if(parc == 2) parv[4] = parv[3];
   parv[3] = nick->user->nick;

   return generic_moduser(nick, chaninfo, parc == 2 ? 4 : 3, parv);
}

int moduser_autovoice(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
   parv[2] = "AUTOVOICE";
   if(parc == 2) parv[4] = parv[3];
   parv[3] = nick->user->nick;

   return generic_moduser(nick, chaninfo, parc == 2 ? 4 : 3, parv);
}

