/* src/serveur.c - Fonctions serveur
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "serveur.h"

CVSINFO("$Id: serveur.c,v 1.133 2003/08/05 15:12:56 romexzf Exp $");

struct Link *mainhub = NULL;
int dataQ = 0;
int burst;
int registered;
int want_to_quit;

struct aMsg msgtab[] = {
  {TOKEN_PRIVMSG, m_privmsg},
  {TOKEN_PING, m_ping},
  {TOKEN_NICK, m_nick},
  {TOKEN_JOIN, m_join},
  {TOKEN_PART, m_part},
  {TOKEN_MODE, m_mode},
  {TOKEN_KICK, m_kick},
  {TOKEN_CREATE, m_create},
  {TOKEN_QUIT, m_quit},
  {TOKEN_BURST, m_burst},
  {TOKEN_AWAY, m_away},
  {TOKEN_SERVER, m_server},
#ifdef N2K
  {"SERVER", m_server},
  {"OM", m_mode},
  {"CM", m_clearmode},
#endif
  {TOKEN_SQUIT, m_squit},
  {TOKEN_TOPIC, m_topic},
  {TOKEN_KILL, m_kill},
  {TOKEN_WHOIS, m_whois},
#ifndef N2K
  {TOKEN_SETHOST, m_sethost},
  {TOKEN_SETIDENT, m_setident},
  {TOKEN_ACCOUNT, m_account},
#endif
  {TOKEN_PASS, m_pass},
  {TOKEN_EOB, m_eob},
  {NULL, NULL}
};

#ifndef N2K
/*
 * m_account
 *
 * parv[0] = sender (i don't care about)
 * parv[1] = numeric
 * parv[2] = login
 */
void m_account(int parc, char **parv)
{
	aNick *nick;
	anUser *user;
	char *login = NULL;

	if(parc < 3 || !(nick = num2nickinfo(parv[1]))) return;

	login = parv[2];
	if(*login == ':') login++;

	if(!(user = getuserinfo(login)) || user->n)
	{
		struct Link *links = num2servinfo(parv[0]);
		cswallops("ACCOUNT %s (%s) pour %s depuis %s", user ? "Desynch" : "inconnu", login, nick->nick, links->serv);
		cs_account(nick, NULL); /* desacounter :)*/
		return;
	}

	nick->user = user;
	nick->user->n = nick;
	nick->user->lastseend = time(NULL);
	csntc(nick, "Vous avez t \2ridentifi\2");
#ifdef USE_NICKSERV
	if(IsProtected(user)) del_killinfo(nick, user->nick); /* arret du kill */
#endif
	return;
}
#else

/*
 * m_clearmode
 */
void m_clearmode(int parc, char **parv)
{
	aChan *chan = getchaninfo(parv[1]);
	int clear_op = strchr(parv[2], 'o') ? 1 : 0;
	int clear_voice = strchr(parv[2], 'v') ? 1 : 0;

	if(chan && !IsSuspend(chan))
	{
		aLink *j;
		char mode[30];

		snprintf(mode, sizeof mode, "-%s", parv[2]);
		parse_cmode(chan, mode, NULL, NULL);

		if(!clear_op && !clear_voice) return; /* aucun changement de +o/+v*/
		for(j = chan->members;j;j=j->next)
			 if(!(j->value.j->nick->flag & (N_SERVICE | N_GOD)))
			 {
				 if(clear_op) j->value.j->is_op = 0;
				 if(clear_voice) j->value.j->is_voice = 0;
			 }
	}
	else
	{
		register aNick *nick = nickhead;

		if(!clear_op && !clear_voice) return; /* aucun changement de +o/+v*/
		for(;nick;nick=nick->next)
		{
			register aJoin *join = nick->joinhead;
			for(;join && strcasecmp(join->channel, parv[1]);join=join->next);
			if(join)
			{
				if(clear_op) join->is_op = 0;
				if(clear_voice) join->is_voice = 0;
			}
		}
	}
	return;
}
#endif

/*
 * m_mode
 *
 * parv[0] = who
 * parv[1] = chan
 * parv[2] = modes
 * parv[3->parc-1] = rest
 */
void m_mode(int parc, char **parv)
{
	aNick *nick, *nicktemp = NULL;

	if(parc > 1 && *parv[1] == '#')
	{
		aChan *ch = getchaninfo(parv[1]);
		char *modes = parv[2], *param = NULL, *key = NULL, *limit = NULL;
		anAccess *a = NULL, *a2 = NULL;
		aJoin *join = NULL;
		aLink *j = NULL;
		int what = 1, i = 2, l = 0;

		while (modes && *modes)
		{
			switch (*modes)
			{
				case '+':
					what = 1;
					break;
				case '-':
					what = 0;
					break;
				case 'o':
					do param = parv[++i];
					while(!*param || !param);

					if(ch) /* salon reg */
					{
						if(!what) /* -o */
						{
							if(!strcmp(cs.num, param)) /* deop du CServ */
							{
								csmode(ch, "+o %s", cs.num);
								break;
							}
							/* protection du deop des admins /user auths */
							if((nicktemp = GetMemberIbyNum(ch, param)) && nicktemp->user &&
#ifdef N2K
							(nick = GetMemberIbyNum(ch, parv[0]))
#else
							(nick = GetMemberIbyNick(ch, parv[0]))
#endif
							&& nick != nicktemp && (IsAnAdmin(nicktemp->user) || (a = GetAccessIbyUserI(nicktemp->user, ch->chan))))
							{
								if(!nick->user || (nicktemp->user->level > nick->user->level ||
								(a && AProtect(a) && !ASuspend(a) && (a2 = GetAccessIbyUserI(nick->user, ch->chan))
								&& !ASuspend(a2) && a2->level <= a->level)))
								{
									csmode(ch, "+o %s", nicktemp->numeric);
									break;
								}
							}
						}
						else /* +o */
						{ /* noops ou strictop */
							nicktemp = GetMemberIbyNum(ch, param);
							if((CNoOps(ch) && nicktemp && !(nicktemp->flag & N_OPER))
								|| (CStrictOp(ch) && nicktemp && (!nicktemp->user || !(a = GetAccessIbyUserI(nicktemp->user, ch->chan)) || ASuspend(a))))
							{
								csmode(ch, "-o %s", param);
								break;
							}
						}
						/* mise  jour des modes */
						for(j=ch->members;j && strcmp(j->value.j->nick->numeric, param);j=j->next);
						if(j) j->value.j->is_op = what;

					} /* mise  jour direct dans nick->join */
					else if((nicktemp = num2nickinfo(param)) && (join = getjoininfo(nicktemp, parv[1]))) join->is_op = what;

					break;

				case 'v':
					do param = parv[++i];
					while(!*param || !param);

					if(ch)
					{
						for(j=ch->members;j && strcmp(j->value.j->nick->numeric, param);j=j->next);
						if(j) j->value.j->is_voice = what;
					}
					else if((nicktemp = num2nickinfo(param)) && (join = getjoininfo(nicktemp, parv[1]))) join->is_voice = what;
					break;

				case 'b':
					do param = parv[++i]; /*l on modifie 'i' pour qu'il passe au paramtre de ce mode*/
					while(!*param || !param);

					if(!what || !ch) break;
					if(!CNoBans(ch))
					{
						char h[NUHLEN]; /* 3 = ! @ \0 */
#ifdef N2K
						if(!(nicktemp = GetMemberIbyNum(ch, parv[0]))) break;
#else
						if(!(nicktemp = GetMemberIbyNick(ch, parv[0]))) break;
#endif
						l = nicktemp->user ? ChanLevelbyUserI(nicktemp->user, parv[1]) : 0;
						for(j=ch->members;j;j=j->next)
						{
							nick=j->value.j->nick;
							snprintf(h, NUHLEN, "%s!%s@%s", nick->nick, nick->ident,
#ifdef HAVE_CRYPTHOST
							nick->crypt
#else
							nick->host
#endif
							);
							if(match(param, h) || !nick->user) continue;
							if((a = GetAccessIbyUserI(nick->user, ch->chan)) && AProtect(a) && !ASuspend(a)) break;
							else a = NULL;
						}
					}
					if(CNoBans(ch) || ((ch->banlevel > l || a) && (!nicktemp->user || !IsAnAdmin(nicktemp->user))))
					{
						csmode(ch, "-b %s", param);
						break;
					}
					break;
				case 'l':
					if(what && is_num(parv[++i])) limit = parv[i];
					break;
				case 'k':
					key = parv[++i];
					break;
			}
			modes++;
		}
		if(ch)
		{ /*test du ChangeModesLevel (et action..), un peu lourd mais efficace*/
#ifdef N2K
			if((nicktemp = GetMemberIbyNum(ch, parv[0])) &&
#else
			if((nicktemp = GetMemberIbyNick(ch, parv[0])) &&
#endif
				ch->cml && nicktemp->user && !IsAnAdmin(nicktemp->user) &&
				(!(a = GetAccessIbyUserI(nicktemp->user, ch->chan)) || ASuspend(a) || a->level < ch->cml))
			{
				char new[30] = {0}, tmp[20];
				int f = 0, nk = 0;
				modes = parv[2];
				while (modes && *modes)
				{
					if(*modes != 'b' && *modes != 'v' && *modes != 'o')
					{
						if(*modes == '+') new[f] = '-';
						else if(*modes == '-') new[f] = '+';
						else if(*modes == 'k') nk = 1;
						else new[f] = *modes;
						f++;
					}
					modes++;
				}
				snprintf(tmp, 20, "%d", ch->limit);
				if(new[1] != '\0') putserv("%s " TOKEN_MODE " %s %s %s\r\n", cs.num, ch->chan, new,
					((ch->mode & C_MLIMIT) && (limit || strchr(new, 'l'))) ? tmp : "");
				if(nk) putserv("%s " TOKEN_MODE " %s %ck %s\r\n", cs.num, ch->chan, (ch->mode & C_MKEY) ? '+' : '-', (ch->mode & C_MKEY) ? ch->key : key);
			}
			else parse_cmode(ch, parv[2], limit, key);
		}
	}
	else if((nick = getnickbynick(parv[1]))) parse_umode(nick, parv[2]);
	return;
}

/*
 * m_part
 *
 * parv[0] = nick
 * parv[1] = chans
 * parv[2] = raison
 */
void m_part(int parc, char **parv)
{
	char *p;
	aNick *nick = NULL;

	if(parc < 2) return;
#ifdef N2K
	nick = num2nickinfo(parv[0]);
#else
	nick = getnickbynick(parv[0]);
#endif
	if(!nick)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: serveur.c -> m_part -> nick == NULL\n");
		cswallops("ERREUR : Part d'un nick inconnu : %s", parv[0]);
		return;
	}
	for(p = strtok(parv[1], ",");p;p = strtok(NULL, ",")) del_join(nick, p);
	return;
}

/*
 * m_kick
 *
 * parv[0] = kickeur fou
 * parv[1] = chan
 * parv[2] = num victime
 * parv[3] = raison
 */
void m_kick(int parc, char **parv)
{
	aNick *k, *v;
	aChan *ch;
	aJoin *j = NULL;
	anAccess *a1 = NULL, *a2 = NULL;

	if(parc < 3) return;
	if(!strcmp(cs.num, parv[2]) && !strcasecmp(parv[1], bot.pchan))
	{
        putserv("%s %s %s\r\n", cs.num, TOKEN_JOIN, parv[2]);
		putserv("%s %s %s +o %s\r\n", cs.num, TOKEN_MODE, parv[2], cs.num);
		return;
	}
	if(!(ch = getchaninfo(parv[1]))) return;
	if(!strcmp(cs.num, parv[2])) /*on sait jamais*/
	{
		csjoin(ch);
		return;
	}

	if(!(v = GetMemberIbyNum(ch, parv[2]))) return;

#ifdef N2K
	k = GetMemberIbyNum(ch, parv[0]);
#else
	k = GetMemberIbyNick(ch, parv[0]);
#endif
	del_join(v, parv[1]);

	if(v->user && k && k != v /* gnrique*/
		&& ((IsAnAdmin(v->user) && (!k->user || k->user->level < v->user->level))/* v admin >*/
			|| ((a1 = GetAccessIbyUserI(v->user, parv[1])) && !ASuspend(a1)/* OU v a un accs valide*/
				 && (!k->user  /* et k non logu*/
					 || (!IsAnAdmin(k->user)/* si auth mais PAS admin */
				 		&& (!(a2 = GetAccessIbyUserI(k->user, parv[1])) || ASuspend(a2)/* OU n'a pas d'accs valide*/
							|| (AProtect(a1) && a2->level <= a1->level))))))) /* OU v protect ET k <= v*/
	{
		csmode(ch, "-o %s", k->numeric);/*pas le droit de kick > dop*/
		if((j = getjoininfo(k, ch->chan))) j->is_op = 0;
	}
	return;
}

/*
 * m_join
 *
 * parv[0] = nick
 * parv[1] = chans
 */
void m_join(int parc, char **parv)
{
	char *p;
#ifdef N2K
	aNick *nick = num2nickinfo(parv[0]);
#else
	aNick *nick = getnickbynick(parv[0]);
#endif

	if(parc < 2) return;
	if(!nick)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: serveur.c -> m_join -> nick == NULL\n");
		cswallops("ERREUR : Join d'un nick inconnu : %s", parv[0]);
		return;
	}
	if(*parv[1] == '0') /* partall !!*/
	{
		aJoin *join = nick->joinhead, *jt;
		for(;join;join = jt)
		{
			jt = join->next;
			del_join(nick, join->channel);
		}
	}
	else for(p = strtok(parv[1], ",");p;p=strtok(NULL, ",")) add_join(nick, p, 0, 0, time(NULL), 0, NULL);
	return;
}

/*
 * m_create
 *
 * parv[0] = num
 * parv[1] = chan
 * parv[2] = timestamp
 */
void m_create(int parc, char **parv)
{
	char *p;
	aNick *nick;

	if(parc < 3) return;
	if(!(nick = num2nickinfo(parv[0])))
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: serveur.c -> m_create -> nick == NULL\n");
		cswallops("ERREUR : create d'un nick inconnu : %s", parv[0]);
		return;
	}

	for(p = strtok(parv[1], ",");p;p = strtok(NULL, ",")) add_join(nick, p, 1, 0, atol(parv[2]), 0, NULL);
	return;
}

#ifndef N2K
/*
 * m_setident
 * parv[0] = num
 * parv[1] = host
 */
void m_setident(int parc, char **parv)
{
	aNick *who = num2nickinfo(parv[0]);

	if(parc < 2 || !who) return;
	Strncpy(who->ident, parv[1], USERLEN);
	return;
}

/*
 * m_sethost
 *
 * parv[0] = num
 * parv[1] = host
 */
void m_sethost(int parc, char **parv)
{
	aNick *who = num2nickinfo(parv[0]);

	if(parc < 2 || !who || *parv[1] == 0 || strchr(parv[1], ':') || !strchr(parv[1], '.')) return;

#ifdef HAVE_CRYPTHOST
	Strncpy(who->crypt, parv[1], HOSTLEN);
#else
	Strncpy(who->host, parv[1], HOSTLEN);
#endif
	return;
}
#endif

/*
 *m_nick
 *
 * parv[1] = nick
 * parv[2] = hop
 * parv[3] = timestamp
 * parv[4] = username
 * parv[5] = host
 *  ==[ user a un/des mode(s) | user a pas de mode ]==
 * parv[6] = modes | base64
 * parv[7] = base64 | num
 * parv[8] = num | name
 * parv[9] = name |
 */
void m_nick(int parc, char **parv)
{
	char *nick, *host, *num = NULL, *ac = NULL, *spoof = NULL;
	aNick *who = NULL;
	anUser *user;
	struct Link *serv;
	struct badnicks *badnick;

	if(parc > 7)
	{  /*Nouveau nick*/
		host = parv[5];
		nick = parv[1];
						/*  		ident	host	base64			num				ptr to serv						real-name		timestamp*/
		who = add_nickinfo(nick, parv[4], host, parv[parc-3], parv[parc-2], (serv = num2servinfo(parv[parc-2])), parv[parc-1], atol(parv[3]));
		if(!who)
		{
			putlog(LOG_ERREURS, "ERREUR MAJEURE: m_nick -> add_nickinfo -> NULL\n");
			return;
		}

		if(*parv[6] == '+')
		{
			int i = 7;
			parse_umode(who, parv[6]);
			if(who->flag & N_REG) ac = parv[i++];
			if(who->flag & N_SPOOF) spoof = parv[i++];
		}

#ifdef HAVE_CRYPTHOST
		Strncpy(who->crypt, strchr(host, '.') ? hostprot(host) : host, HOSTLEN); /* cryptage de l'host en md2 ou md5 suivant certains critres*/
#endif
		num = who->numeric;

		if(ac)
		{
			if(!(user = getuserinfo(ac)) || user->n)
			{
				cswallops("ACCOUNT %s (%s) pour %s depuis %s", ac, user ? "Desynch" : "inconnu", who->nick, serv->serv);
				cs_account(who, NULL);
				return;
			}
			who->user = user;
			who->user->n = who;
			who->user->lastseend = time(NULL);
			csntc(who, "Vous avez t \2ridentifi\2");
		}

#ifdef USE_WELCOMESERV /* Envoi du welcome si il est actif et non nul*/
		if(GetConf(CF_WELCOME) && !burst) choose_welcome(who->numeric);
#endif

	}
	else if(parc <= 3)
	{  /* Changement de pseudo */
		nick = parv[1];
		num = parv[0];
		who = num2nickinfo(num);
		if(!who)
		{
			putlog(LOG_ERREURS, "ERREUR MAJEURE: serveur.c -> m_nick -> who == NULL\n");
			cswallops("ERREUR : Changement de pseudo d'un nick inconnu : %s", parv[0]);
			return;
		}
		del_killinfo(who, NULL);
		Strncpy(who->nick, nick, NICKLEN);
	}
	else return;

	if(!who) who = num2nickinfo(num);/*on refait pas la loop*/
	if(!who)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: serveur.c -> m_nick -> who == NULL\n");
		cswallops("ERREUR : NICK d'un nick inconnu : %s", parv[0]);
		return;
	}
	if((badnick = getbadnick(nick)) && !(badnick->flag & CF_NOREG))
	{
		csntc(who, "\2Avertissement\2 : Vous venez de prendre un pseudo illgal");
		if(!GetConf(CF_CHANGENICK))
			csntc(who, "Veuillez changer de pseudo sinon vous serez dconnect dans %d secondes.", kill_interval);
		add_killinfo(who, time(NULL), 1, !GetConf(CF_CHANGENICK));
		return;/*on check meme pas le reste..*/
	}
	if(who->user && !strcasecmp(who->user->nick, nick)) return;/*le mec est auth sur ce nick > exit*/

#ifdef USE_NICKSERV
	if(!(user = getuserinfo(nick))) return;/*le pseudo n'est pas reg, on evite leisregnick > exit*/

	csntc(who, "\2Avertissement\2 : Le pseudo \2%s\2 est enregistr, si c'est le votre, identifiez vous via: /%s %s %s <votre pass>",
		nick, cs.nick, RealCmd("login"), nick);
	if(IsProtected(user))
	{
		if(UPKill(user)) csntc(who, "Veuillez changer de pseudo sinon vous serez dconnect dans %d secondes.", kill_interval);
		else csntc(who, "Veuillez changer de pseudo sinon votre pseudo sera automatiquement chang dans %d secondes.", kill_interval);
		add_killinfo(who, time(NULL), 2, !!UPKill(user));
	}
#endif
	return;
}

/*
 *
 *
 * parv[0] = num
 * parv[1] = chan
 * parv[2] = timestamp
 * parv[3] = modes / users
 * parv[4] = users / bans
 * parv[5] = bans
 */
void m_burst(int parc, char **parv)
{
	aNick *nick = NULL;
	aLink *j = NULL;
	aChan *c = NULL;
	char *modes = NULL, *chan = parv[1], *timestamp = parv[2], *users = NULL, *key = NULL, *limit = NULL;
	char *flags = NULL, *num = NULL/*, *bans = NULL*/;
	int op = 0, voiced = 0, i = 4;
	long il = atol(timestamp);

	if((parc > 3 && *parv[3] == '%') || parc < 4) return;

	if(parc > 4 && *parv[3] == '+')
	{  /* channel has mode(s) */
		modes = parv[3];
		if(strchr(modes, 'l')) limit = parv[i++];
		if(strchr(modes, 'k')) key = parv[i++];
		users = parv[i];
		/*if(parc > i) bans = parv[++i];*/
	}
	else users = parv[3];

	if(!burst)
	{
		if((c = getchaninfo(chan)) && (!c->chanstamp || il < c->chanstamp))
		{ /* les users du chan vont se faire deop et les modes rinitialiss comme le veut le serveur qui envoi le burst*/
			parse_cmode(c, modes, limit, key);
			putserv("%s " TOKEN_MODE " %s +o %s\r\n", bot.servnum, c->chan, cs.num);
			c->chanstamp = il;
			/* le pb ici c'est qu'on ne peut dterminer un user qui s'est fait deop UNIQUEMENT sur les
			chans regs, donc si un mec tait op sur un chan non reg, que le timestamp le fasse se deop,
			et qu'il veut reg le chan, il pourra sans etre op (bien qu'il l'tait tout de mme avant)
			Toute faon  part a on s'en fou vu que les chans sont pas reg on en a rien  battre */
			for(j=c->members;j;j=j->next) j->value.j->is_op = j->value.j->is_voice = 0;
		}
	}
	else
	{
		if(!strcasecmp(bot.pchan, chan))
		{
#ifdef USE_WELCOMESERV
			putserv("%s " TOKEN_BURST " %s %lu +mints %s:ov,%s\r\n", bot.servnum, bot.pchan, il, ws.num, cs.num);
#else
			putserv("%s " TOKEN_BURST " %s %lu +mints %s:ov\r\n", bot.servnum, bot.pchan, il, cs.num);
#endif
			burst = -burst; /* pour remplacer le 'gotcs' local, sans rajouter de global var*/
		}
		else if((c = getchaninfo(chan)) && !IsSuspend(c) && !CJoined(c))
		{
			putserv("%s " TOKEN_BURST " %s %lu %s %s:o\r\n", bot.servnum, c->chan, il, c->defmodes, cs.num);
			parse_cmode(c, modes, limit, key);
			if(CLockTopic(c))
			{
				if(c->topic[0] != '\0') cstopic(c, c->topic);
				else if(c->deftopic[0] != '\0') cstopic(c, c->deftopic);
			}
			SetCJoined(c);
			c->chanstamp = il;
		}
	}

	while((num = strtok(users, ",")))
	{
		if((flags = strchr(num, ':')))
		{
			op = voiced = 0;
			*flags = '\0';
			while (*(++flags))
			if (*flags == 'o') op = 1;
			else if (*flags == 'v') voiced = 1;
			else break;
		}
		if((nick = num2nickinfo(num))) add_join(nick, chan, op, voiced, il, 1, c);
		users = NULL;

	}
 /* if(bans)
  	{
  	  bans += 2;
	  pv=bans;
	  while((mask=strtok(pv, " ")))
	  {
	        add_ban(chan, mask, "Aucune", "MODE", time(NULL), 0, 0);
	  	pv=NULL;
	  }
 	 }*/
	return;
}

/*
 * m_quit
 *
 * parv[0] = nick
 * parv[1] = msg
 */
void m_quit(int parc, char **parv)
{
#ifdef N2K
	aNick *nick = num2nickinfo(parv[0]);
#else
	aNick *nick = getnickbynick(parv[0]);
#endif
	if(!nick)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: serveur.c -> m_quit -> nick == NULL\n");
		cswallops("ERREUR : Quit d'un nick inconnu : %s", parv[0]);
		return;
	}
	del_nickinfo(nick);
	return;
}

/*
* m_kill
* Nettoyage des vars
*/

void m_kill(int parc, char **parv)
{
	aNick *nick = num2nickinfo(parv[1]);

	if(!nick)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: serveur.c -> m_kill -> nick == NULL\n");
		cswallops("ERREUR : Kill d'un nick inconnu : %s", parv[1]);
		return;
	}
	del_nickinfo(nick);
	return;
}

/*
 * m_away
 *
 * parv[0] = who
 * parv[1-parc] = raison
 */
void m_away(int parc, char **parv)
{
#ifdef N2K
    aNick *nick = num2nickinfo(parv[0]);
#else
    aNick *nick = getnickbynick(parv[0]);
#endif
	if(!nick) return;
	if(parc > 1 && strlen(parv[1]) > 0) nick->flag |= N_AWAY;
	else
	{
		nick->flag &= ~N_AWAY;
#ifdef USE_MEMOSERV
		if(nick->user) show_notes(nick);
#endif
	}
	return;
}

/*
 * m_whois
 *
 * parv[0] = who
 * parv[1] = ma num
 * parv[2] = pseudo du robot
 */
void m_whois(int parc, char **parv)
{
	if(!strcasecmp(parv[2], cs.nick))
		putserv("%s 311 %s %s %s %s * :%s\r\n", bot.servnum, parv[0], cs.nick, cs.ident, cs.host, cs.name);
#ifdef USE_WELCOMESERV
	else if(!strcasecmp(parv[2], ws.nick))
		putserv("%s 311 %s %s %s %s * :%s\r\n", bot.servnum, parv[0], ws.nick, ws.ident, ws.host, ws.name);
#endif
	else
	{
		char insulte[100];
		int i;
		aNick *tmp = getnickbynick(parv[2]);
		if(!tmp) return;

		i = rand()%10;
		if(!i) strcpy(insulte, "a une face de cul de babouin !");
		else if(i == 1) strcpy(insulte, "est une lamouze !");
		else if(i == 2) strcpy(insulte, "est transsexuel !");
		else if(i == 3) strcpy(insulte, "a un QI ngatif !");
		else if(i == 4) strcpy(insulte, "a la diahrre !");
		else strcpy(insulte, "est completement dbile !");

#ifdef HAVE_CRYPTHOST
		putserv("%s 311 %s %s %s %s * :%s\r\n", bot.servnum, parv[0], parv[2], tmp->ident, (tmp->user && tmp->flag & N_CRYPT) ? "hidden" : tmp->crypt, tmp->name);
#else
		putserv("%s 311 %s %s %s %s * :%s\r\n", bot.servnum, parv[0], parv[2], tmp->ident, (tmp->user && tmp->flag & N_CRYPT) ? "hidden" : tmp->host, tmp->name);
#endif
		putserv("%s 312 %s %s %s :%s\r\n", bot.servnum, parv[0], parv[2], tmp->serveur?tmp->serveur->serv:bot.server, tmp->name);
		putserv("%s 390 %s %s :%s\r\n", bot.servnum, parv[0], parv[2], insulte);
		if(tmp->flag & N_OPER) putserv("%s 313 %s %s :est un IRC Oprateur tyrannique et pervers\r\n", bot.servnum, parv[0], parv[2]);
		putserv("%s 318 %s %s :Fin de la commande /WHOIS\r\n", bot.servnum, parv[0], parv[2]);
		return;
	}
	putserv("%s 312 %s %s %s :%s\r\n", bot.servnum, parv[0], parv[2], bot.server, bot.name);
	putserv("%s 313 %s %s :est un IRC Oprateur - Service\r\n", bot.servnum, parv[0], parv[2]);
	putserv("%s 317 %s %s %lu %lu :Secondes d'idle, secondes d'uptime\r\n", bot.servnum, parv[0], parv[2], (time(NULL) - bot.uptime), bot.uptime);
	putserv("%s 318 %s %s :Fin de la commande /WHOIS\r\n", bot.servnum, parv[0], parv[2]);
	return;
}

void m_topic(int parc, char **parv)
{
	aChan *chan = getchaninfo(parv[1]);

	if(!chan) return;
	if(CLockTopic(chan))
	{
		if(chan->topic[0] != '\0') cstopic(chan, chan->topic);
		else cstopic(chan, chan->deftopic);
	}
#ifdef N2K
	else Strncpy(chan->topic, parv[3], TOPICLEN);
#else
	else Strncpy(chan->topic, parv[2], TOPICLEN);
#endif
	return;
}

/*
 * m_eob
 *
 * parv[0] = who
 * parv[1] = me :)
 */
void m_eob(int parc, char **parv)
{
	struct Link *serv = NULL;

	if (!(serv = num2servinfo(parv[0])) || !(serv->flag & ST_BURST))
	{
		cswallops("ERREUR: EOB incorrect serveur inconnu/synch");
		return ;
	}
	serv->flag &= ~ST_BURST;
	serv->flag |= ST_ONLINE;
	if(serv == mainhub)
	{
		aChan *chan;
		time_t current = time(NULL);

#ifdef USE_WELCOMESERV
		if(burst > 0) putserv("%s " TOKEN_BURST " %s %lu +mints %s:ov,%s\r\n", bot.servnum, bot.pchan, current, cs.num, ws.num);
#else
		if(burst > 0) putserv("%s " TOKEN_BURST " %s %lu +mints %s:ov\r\n", bot.servnum, bot.pchan, current, cs.num);
#endif
		else if(burst < 0) burst = -burst;

		for(chan = chanhead;chan;chan = chan->next)
			if(!CJoined(chan) && !IsSuspend(chan))
			{
				char tmp[30] = {0}, *p = NULL, *key = NULL, *limit = NULL;

				putserv("%s " TOKEN_BURST " %s %lu %s %s:o\r\n", bot.servnum, chan->chan, current, chan->defmodes, cs.num);
				if(chan->topic[0] != '\0') cstopic(chan, chan->topic);
				else if(chan->deftopic[0] != '\0') cstopic(chan, chan->deftopic);
				SetCJoined(chan);
				chan->chanstamp = current;

				if(chan->defmodes[0] != '\0')
				{
					strcpy(tmp, chan->defmodes);
					if(!(p = strtok(tmp, " "))) continue;
					while(*p && *p != ' ')
					{
						if(*p == 'k') key = strtok(NULL, " ");
						if(*p == 'l') limit = strtok(NULL, " ");
						p++;
					}
					parse_cmode(chan, tmp, limit, key);
				}
			}

		putserv("%s " TOKEN_EOB "\r\n", bot.servnum);
		putserv("%s " TOKEN_EOBACK "\r\n", bot.servnum);

		if(dataQ)
		{
			struct timeval tv;
			float tt;

			gettimeofday(&tv, NULL);/*un peu de stats sur le burst pour *fun* */
			tt = (tv.tv_usec - burst) / 1000000.0;
			if(tt < 0) tt = -tt;
			cswallops("Synchronisation de %d bytes en %12.6f s (Taux %12.6f Ko/s)", dataQ, tt,(float) (dataQ / 1024.0) / tt);
		}
		burst = 0;
	}
	alarm(ALARMINTERVAL);
	return;
}

static void ClearNicksbyServ(struct Link *links)
{
	register aNick *nick = nickhead, *nt = NULL;

	for(;nick;nick=nt)
	{
		nt = nick->next;
		if(nick->serveur == links) del_nickinfo(nick);
	}
}

void m_server(int parc, char **parv)
{
	struct Link *links, *last;

	if(!(links = calloc(1, sizeof(struct Link))))
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un Link %s !\n", parv[1]);
		return;
	}

	strncpy(links->serv, parv[1], 150);
#ifdef N2K
	strncpy(links->num, parv[6], 2);
#else
	strncpy(links->num, parv[6], 1);
#endif
	links->flag = 0;
	links->next = NULL;

	if(atoi(parv[2]) == 1)/*1erhub*/
	{
		links->hub = NULL;
		links->last = NULL;
		mainhub = links;
		registered = 1;
	}
	else
	{
		last = num2servinfo(parv[0]);
		links->hub = last; /* ptr vers son Hub */
		last->flag |= ISHUB;
		for(last=mainhub;last->next;last=last->next); /* get last ptr*/
		last->next = links; /* chainage positif */
		links->last = last; /* chainage ngatif */
	}

	if (*parv[5] == 'J') links->flag |= ST_BURST;
	else links->flag |= ST_ONLINE;
	return;
}

static void NullHub(struct Link *hub)
{
	struct Link *l = hub;
	for(;l;l=l->next) if(l->hub == hub) l->hub = NULL;
}

void m_squit(int parc, char **parv)
{
	struct Link *links = mainhub, *last = NULL, *lt = NULL;

	/*nettoyage des serveurs en "dessous" (dont il est Hub) */
	for(;links && strcasecmp(parv[1], links->serv);links=links->next);
	if(links)
	{
		if(links == mainhub) /* squit de l'uplink, tentative de reconnexion.. */
		{
			for(;links;links = lt)
			{
				lt = links->next;
				free(links);
			}
			mainhub = NULL;
			return;
		}

		if(links->next) links->next->last = links->last;
		links->last->next = links->next;

		if(links->flag & ISHUB)
		{
			NullHub(links); /* s'il est hub ya forcement un leaf (next) apres!? */
			for(last = links->next;last;last = lt)
			{
				lt = last->next;
				if(!last->hub)
				{
					ClearNicksbyServ(last);
					if(last->flag & ISHUB) NullHub(last);/*  on met les ptr vers ce hub (leaf du premier) a NULL*/
					last->next->last = last->last;/*		 ses leafs seront del lors de la suite de la boucle*/
					last->last->next = last->next;
					free(last);
					last = NULL;
				}
			}
		}
		ClearNicksbyServ(links);
		free(links);
		links = NULL;
	}

	return;
}

void m_pass(int parc, char **parv)
{
	struct timeval tv;
	time_t current = time(NULL);

	gettimeofday(&tv, NULL);
	burst = tv.tv_usec;

	putlog(LOG_PARSES, "Connect au serveur %s\n", bot.ip);
	registered = 0;
	bot.lasttime = current;
	bot.lastbytes = 0;
	if(!bot.uptime) bot.uptime = current;
	if(!lastresetlogs) lastresetlogs = current;
	if(!lastresetparse) lastresetparse = current;
	return;
}

void m_ping(int parc, char **parv)
{
#ifdef N2K
	struct timeval tv;
	char *ptr = parv[1], *theirttm = strchr(parv[1], '.');

	if(theirttm && *theirttm == '.') theirttm++;
	if(*ptr == '!') ptr++;
	gettimeofday(&tv, NULL);
/* AsLL */
	putserv("%s " TOKEN_PONG " %s %s %s %d %ld.%d\r\n", bot.servnum, bot.servnum,
			parv[0], ptr, (int) (tv.tv_usec - atol(theirttm)), time(NULL), (int) tv.tv_usec);
#else
    putserv("%s " TOKEN_PONG " %s\r\n", bot.servnum, parv[1]);
#endif
    return;
}

static int exec_cmd(unsigned int i, unsigned int securise, char *ni, int parc, char **parv)
{
	char *tmp3 = NULL, buff[512], tmp[(2 * (NICKLEN + 1)) + 5];
	int len = 0;
	aChan *chaninfo = NULL;
	anAccess *acces = NULL;
	aNick *nick = NULL;
#ifdef HAVE_TRACK
	struct track *track = NULL;
#endif

#ifdef N2K
	if(!(nick = num2nickinfo(ni))) return 0; /* !!!!*/
#else
	if(!(nick = getnickbynick(ni))) return 0; /* !!!!*/
#endif

	if(isignore(nick) || checkflood(nick)) return 0;

#ifdef HAVE_SECURE
	if(cs_cmds[i].secure && !securise && (strcasecmp(cs_cmds[i].realcmd, "memos") || !parc || !strcasecmp(parv[1], "SEND")))
		return csntc(nick, "La commande\2 %s\2 demande des informations prives. Veuillez utiliser \2/%s %s %s\2.",
			cs_cmds[i].cmd, cs.nick, cs_cmds[i].cmd, parv2msg(parc, parv, 1, 400));
#endif

	cs_cmds[i].used++;
	if((cs_cmds[i].flag & CMD_DISABLE))
		return csntc(nick, "\2La commande %s est dsactive.", cs_cmds[i].cmd);

	if(parc < cs_cmds[i].args || (!parc && ChanCmd(i)))
		return syntax_cmd(nick, i);

	if(ChanCmd(i))
	{
		if(*parv[1] != '#' || !(chaninfo = getchaninfo(parv[1])))
			return csntc(nick, NOSUCHAN, parv[1]);

		if(IsSuspend(chaninfo))
			return csntc(nick, "%s est suspendu : %s - %s", parv[1], chaninfo->suspend->raison, chaninfo->suspend->from);

		else if(chaninfo->suspend && chaninfo->suspend->expire >= 0)
		{
			chaninfo->suspend->expire = -1;
			csjoin(chaninfo);
			putlog(LOG_CHANS, "SUSPEND OFF (Auto) %s par %s\n", chaninfo->chan, chaninfo->suspend->from);
		}
	}
	if(cs_cmds[i].level)
	{
		if(!nick->user || !nick->user->n)
			return csntc(nick, "%s : Vous n'tes pas logu ou vous n'avez pas encore enregistr d'Username", pasdeperm);

		if(AdmCmd(i) && nick->user->level < cs_cmds[i].level)
			return csntc(nick, "%s : Vous devez tre Administrateur pour utiliser cette commande", pasdeperm);

		if(ChanCmd(i) && !IsAnAdmin(nick->user))
		{
			if(!(acces = GetAccessIbyUserI(nick->user, parv[1])))
				return csntc(nick, "%s : Vous n'avez pas d'accs sur le salon %s", pasdeperm, parv[1]);

			if(cs_cmds[i].level > acces->level)
				return csntc(nick, "%s : Votre niveau sur %s est trop petit (Vous:\2 %d\2, Commande:\2 %d\2)",
							pasdeperm, parv[1], acces->level, cs_cmds[i].level);

			if(ASuspend(acces))
				return csntc(nick, "%s : Votre accs est suspendu sur %s", pasdeperm, parv[1]);
		}
	}

	sprintf(tmp, " par %s%s%s", nick->nick, nick->user ? "@" : "", nick->user ? nick->user->nick : "");

	buff[0] = '\2';
	strcpy(buff + 1, cs_cmds[i].cmd);
	len = 1 + strlen(cs_cmds[i].cmd);
	buff[len++] = '\2';
	buff[len++] = ' ';
	buff[len] = '\0';
	if(ChanCmd(i))
	{
		strcpy(buff + len, chaninfo->chan);
		len += strlen(chaninfo->chan);
		buff[len++] = ' ';
		buff[len] = '\0';
		if(parc > 1) strcpy(buff + len, (tmp3 = parv2msg(parc, parv, 2, 400)));
	}
	else
	{
		if((cs_cmds[i].secure == 1 && parc) || (parc >= 2
			&& ((!strcasecmp(cs_cmds[i].realcmd, "USER") && !strcasecmp(parv[1], "NEWPASS"))
			|| (!strcasecmp(cs_cmds[i].realcmd, "SET") && !strcasecmp(parv[1], "PASS")))))
				strcpy(buff + len, (tmp3 = parv[1]));
		else if(cs_cmds[i].secure != 2 && parc && cs_cmds[i].args)
			strcpy(buff + len, (tmp3 = parv2msg(parc, parv, 1, 400)));
	}
	strcat(buff, tmp);

#ifdef HAVE_TRACK
	if(nick->user && (track = istrack(nick->user))) csntc(track->tracker, "[\2Track\2] %s", buff);
#endif
	putchan(buff);

#ifdef SQLLOG
	if(cs_cmds[i].level)
	{
		if(ChanCmd(i))
		{
			if(tmp3) *(tmp3 + 254) = 0;
			sql_query("INSERT INTO chancmdslog (user, chan, TS, cmd, log) VALUES ('%s', '%s', UNIX_TIMESTAMP(), '%s', '%s');", nick->user->nick, chaninfo->chan, cs_cmds[i].realcmd, tmp3);
		}
		else sql_query("INSERT INTO usercmdslog (user, TS, cmd, log) VALUES ('%s', UNIX_TIMESTAMP(), '%s', '%s');", nick->user->nick, cs_cmds[i].realcmd, NONE(tmp3));
	}
#else
	putlog(LOG_CMDS, "%s\n", buff);
#endif
	cs_cmds[i].func(nick, chaninfo, parc, parv);/*permet de passer la struct chaninfo (economie de loop)*/
	return 1;
}

/*
 * m_privmsg
 *
 * parv[0] = envoyeur
 * parv[1] = destinataire (pseudo du robot, set dans botnick)
 * parv[2] = message reu.
 *
 * pparv[0] = nom de la commande
 * pparv[pparc] = arguments de la commande
 * ATTENTION: pparc est le nb d'arg  partir de *parv[1]*
 */
void m_privmsg(int parc, char **parv)
{
	char *tmp, *cmd = NULL, *botnick = NULL, *pparv[250];
	aNick *ni = NULL;
	unsigned int pparc = 0, securise = 0;
	int i;

	if(*parv[1] != '#')
	{  /* Message envoy  un robot*/

		pparv[1] = NULL;

		if(strchr(parv[1], '@'))
		{
			botnick = strtok(parv[1], "@");
			securise = 1;
		}
		else botnick = parv[1];

		if(!*parv[2] || *parv[2] == ' ' || (strcmp(botnick, cs.num) && strcasecmp(botnick, cs.nick))) return;

		pparv[0] = cmd = (tmp = strtok(parv[2], " "));
		if(!tmp) return;

		if((i = GetCurCmdIndexbyName(cmd)) < 0)
		{
#ifdef N2K
			if(*cmd != '\1' && (ni = num2nickinfo(parv[0]))) csntc(ni, "La commande\2 %s\2 n'existe pas.", cmd);
#else
			if(*cmd != '\1' && (ni = getnickbynick(parv[0]))) csntc(ni, "La commande\2 %s\2 n'existe pas.", cmd);
#endif
			return;
		}

#ifdef N2K
		if(premiere && strcasecmp("REGISTER", cmd) && (ni = num2nickinfo(parv[0])))
#else
		if(premiere && strcasecmp("REGISTER", cmd) && (ni = getnickbynick(parv[0])))
#endif
		{
			csntc(ni, "%s n'a pas encore t enregistr", cs.nick);
			return;
		} /* on ne parse que si la commande existe */

		while((tmp = strtok(NULL, " "))) pparv[++pparc] = tmp;

		exec_cmd(i, securise, parv[0], pparc, pparv);
		return;
	}
	else
	{  /* Message envoy sur un salon (pour chanserv)*/
		if(parv[2] && *parv[2] == bot.cara && *++parv[2] && *parv[2] != ' ')
		{
			pparv[0] = cmd = (tmp = strtok(parv[2], " "));
			if(!tmp) return;

#ifdef N2K
			if(premiere && strcasecmp("REGISTER", cmd) && (ni = num2nickinfo(parv[0])))
#else
			if(premiere && strcasecmp("REGISTER", cmd) && (ni = getnickbynick(parv[0])))
#endif
			{
				csntc(ni, "%s n'a pas encore t enregistr", cs.nick);
				return;
			}

			if((i = GetCurCmdIndexbyName(cmd)) < 0) return;

			tmp = strtok(NULL, " "); /* on traite le premier arg aprs la commande*/
			if(ChanCmd(i) && (!tmp || *tmp != '#')) pparv[++pparc] = parv[1]; /* si ce n'est pas un salon*/
			if(tmp) pparv[++pparc] = tmp;									/* on prend le salon o elle est tape*/

			for(tmp = strtok(NULL, " ");tmp;tmp = strtok(NULL, " ")) pparv[++pparc] = tmp;

			exec_cmd(i, securise, parv[0], pparc, pparv);
			return;
		}
	}
	return;
}

static void parse_this(char *msg)
{
	int parc = 0, i = 0;
	char *cmd = NULL;
	char *parv[MAXPARA + 2]; /* leave room for prefix and end-of-params NULL */
#ifdef DEBUG
	char logg[513];
	strncpy(logg, msg, 512);
	logg[512] = 0;
#endif

	if(!registered) parv[parc++] = bot.server;
#ifndef N2K
	if(*msg == ':') msg++;
#endif
	while(*msg)
	{
		while(*msg == ' ') *msg++ = 0;/* on supprime les espaces pour le dcoupage*/
		if(*msg == ':') /* last param*/
		{
			parv[parc++] = msg + 1;
			break;
		}
		if(!*msg) break;

		if(parc == 1 && !cmd) cmd = msg;/* premier passage  parc 1 > cmd*/
		else parv[parc++] = msg;
		while(*msg && *msg != ' ') msg++;/*on laisse passer l'arg..*/
	}
	parv[parc] = NULL;

#ifdef DEBUG /*etical loging*/
	if(parc <= 1 || ((strcasecmp(cmd, TOKEN_PRIVMSG) || *parv[1] != '#')
		&& strcasecmp(cmd, TOKEN_PING) && strcasecmp(cmd, TOKEN_PONG)))
#ifdef SQLLOG
			sql_query("INSERT INTO debuglog VALUES ('%s', UNIX_TIMESTAMP(), 'R');", logg);
#else
			putlog(LOG_PARSES, "R - %s\n", logg);
#endif
#endif

	for(;msgtab[i].cmd != NULL;i++)
		if (!strcasecmp(msgtab[i].cmd, cmd))
		{
			msgtab[i].func(parc, parv);
			return;/*on sait jamais */
		}
}

int init_bot(int ip, int port)
{
	int sock, r;
	time_t LTS = time(NULL);
	struct sockaddr_in fsocket;
	aNick *nick;

	sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (sock < 0) return -1;

	fsocket.sin_family = AF_INET;
	fsocket.sin_addr.s_addr = ip;
	fsocket.sin_port = htons(port);
	r = connect(sock, (struct sockaddr *) &fsocket, sizeof(fsocket));

	if (r == -1)
	{
		close(sock);
		return -1;
	}

	bot.sock = sock;
	putserv("PASS %s\r\n", bot.pass);
#ifdef N2K
	putserv("SERVER %s 1 %u %u J10 %sA]] +s :%s\r\n", bot.server, STS, LTS, bot.servnum, bot.name);
#else
	putserv("SERVER %s 1 %u %u J10 %sA] :%s\r\n", bot.server, STS, LTS, bot.servnum, bot.name);
#endif

	putserv("%s " TOKEN_NICK " %s 1 %u %s %s %s B]AAAB %s :%s\r\n", bot.servnum, cs.nick, LTS, cs.ident, cs.host, cs.mode, cs.num, cs.name);

	nick = add_nickinfo(cs.nick, cs.ident, cs.host, "B]AAAB", cs.num, NULL, cs.name, LTS);

	parse_umode(nick, cs.mode);
#ifdef HAVE_CRYPTHOST
	strcpy(nick->crypt, cs.host);
#endif

#ifdef USE_WELCOMESERV
	putserv("%s " TOKEN_NICK " %s 1 %u %s %s %s B]AAAB %s :%s\r\n", bot.servnum, ws.nick, LTS, ws.ident, ws.host, ws.mode, ws.num, ws.name);
	nick = add_nickinfo(ws.nick, ws.ident, ws.host, "B]AAAB", ws.num, NULL, ws.name, LTS);

	parse_umode(nick, ws.mode);
#	ifdef HAVE_CRYPTHOST
	strcpy(nick->crypt, ws.host);
#	endif
#endif

	return sock;
}

void run_bot(int sock)
{
	char tbuf[513] = {0};
	int t = 0;
#ifndef TESTBOT
	int r = 0;
	time_t next_ping = (time(NULL) + PING_FREQ), waitping = 0;
	fd_set rfds;
	char c;
#endif

	if (sock < 0 ) return;

#ifdef TESTBOT

	bot.sock = sock;
	w2c_setnonblocking(bot.sock);
	if(sock > highsock) highsock = sock;
	FD_SET(sock, &master);

#else

	FD_ZERO(&rfds);
	FD_SET(sock, &rfds);

#endif

	while (!want_to_quit)
	{

#ifdef TESTBOT
		if(!bot.sock) break;
		if(w2c_checksocks())
		{
			int i = 0, r;

			for(;i <= highsock;i++)
			{
				if(!FD_ISSET(i, &socks)) continue;
				if(i == bot.sock)
				{
					char buf[513] = {0}, *ptr = buf;

					r = recv(i, buf, sizeof buf, 0);
					if (r < 0)
					{
						bot.sock = 0;
						close(i);
						FD_CLR(i, &socks);
					}
					buf[r] = '\0';
					while(*ptr)
					{
						/* lecture du buffer de recv(), puis dcoupage selon les \r\n et,
						enfin on garde en mmoire dans tbuf le dbut d'un paquet IRC s'il
						est incomplet. au select suivant les bytes lui appartenant lui seront
						ajoute grace  t, index permanent.       packet1\r\npacket2\r\n  */
						tbuf[t++] = *ptr;
						if(*ptr == '\n')
						{
							dataQ += t;/*comptage des bytes lues*/
							bot.lastbytes += t;
							if(tbuf[t-2] == '\r') t -= 2;
							else t--;
							tbuf[t] = '\0';
							parse_this(tbuf);
							t = 0;
						}
						ptr++;
					}
				}
				else if(i == bind_sock)
				{
					int newfd;
					unsigned int addrlen;
					char ip[16] = {0};
					struct sockaddr_in remoteaddr;
					struct w2c_clientinfo *cli;
					w2c_hallow *hallow;

					addrlen = sizeof(remoteaddr);
					if ((newfd = accept(bind_sock, (struct sockaddr *)&remoteaddr, &addrlen)) == -1) continue;
					else
					{
						strcpy(ip, inet_ntoa(remoteaddr.sin_addr));
						for(hallow = w2c_hallowhead;hallow && match(hallow->host,ip);hallow=hallow->next);
						if(!hallow) continue; /*non autoris*/
						CONtotal++;
						FD_SET(newfd, &master);
						if (newfd > highsock) highsock = newfd;
						cli = w2c_addclient(newfd, ip);
						w2c_sendrpl(cli, RPL_000); /*Hello World!*/
					}
				}
				else
				{
					char buf[1024] = {0};
					struct w2c_clientinfo *cli;

					if ((r = recv(i, buf, sizeof buf, 0)) <= 0) w2c_delclient((cli = w2c_findclient(i)));
					else
					{
						WEBtrafficDL += r;
						cli = w2c_findclient(i);
						/* Faire une boucle de strtok comme pour le parse ircsock */
						w2c_parsemsg(cli, buf);
					}
				}

			}
			FD_ZERO(&socks);
		}
#else

		if (!FD_ISSET(sock, &rfds))
		{
			if (next_ping <= time(NULL))
			{
				if(!waitping)
				{
					putserv("%s " TOKEN_PING " :%s\n", bot.servnum, bot.server);
					next_ping = time(NULL) + PING_FREQ;
					waitping = 1;
				}
				else
				{
					printf("Pas de rponse pour %s\n", bot.ip);
					close(sock);
					return;
				}
			}
			continue;
		}
		else
		{
			r = recv(sock, &c, 1, 0);
			if (r <= 0 && errno != EINTR) break;

			next_ping = time(NULL) + PING_FREQ;
			waitping = 0;
			tbuf[t++] = c;
			if (c == '\n')
			{
				dataQ += t;/*comptage des bytes lues*/
				bot.lastbytes += t;
				if(tbuf[t-2] == '\r') t -= 2;
				else t--;
				tbuf[t] = '\0';
				parse_this(tbuf);
				t = 0;
			}
		}

#endif

	}
	if (want_to_quit)
	{
		close(sock);
#ifdef TESTBOT
		w2c_closesocks();
#endif
	}
	return;
}
