/* src/welcome.c - Diverses commandes sur le module welcome
 * Copyright (C) 2002-2003 Inter System (Progs@Inter-System.Net)
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <main.h>

CVSINFO("$Id: welcome.c,v 1.11 2003/07/23 14:23:19 romexzf Exp $");

#ifdef USE_WELCOMESERV

extern int ConfFlag;
extern char user_motd[];
extern char admin_motd[];
extern struct welcomeinfo *welcomehead;

int set_welcome(aNick *, aChan *, int, char **);
int add_welcome(aNick *, aChan *, int, char **);
int del_welcome(aNick *, aChan *, int, char **);
int list_welcome(aNick *, aChan *, int, char **);
void send_welcome(const char *, struct welcomeinfo *);
void choose_welcome(const char *);
void add_loadwelcome(int, char *);

/*
 * set_welcome
 *
 * parv[1] = on/off
 */
int set_welcome(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(!parc) ToggleFlag(&ConfFlag, CF_WELCOME);
	else
	{
		if(!strcasecmp(parv[1], "ON")) ConfFlag |= CF_WELCOME;
		else if(!strcasecmp(parv[1], "OFF")) ConfFlag &= ~CF_WELCOME;
		else return csntc(nick, "La valeur doit tre ON ou OFF");
	}
	csntc(nick, "Le welcome est maintenant %s", GetConf(CF_WELCOME) ? "activ" : "desactiv");
	write_welcome();
	return 1;
}

/* /z addwelcome <level> <message> */

int add_welcome(aNick *nick, aChan *c, int parc, char **parv)
{
	struct welcomeinfo *new, *w;
	int level;

	if(!is_num(parv[1])) return csntc(nick, "Le niveau doit tre un nombre.");

	level = atoi(parv[1]);
	if(level < 1 || level > 100)
		return csntc(nick, "Le niveau doit etre compris entre 1 et 100");

	new = calloc(1, sizeof(struct welcomeinfo));
   	if(!new)
   	{
	   	putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour welcomeinfo !\n");
	   	return 0;
   	}

	w = welcomehead;
	while(w)
	{
		if(!w->next) break;
		else w = w->next;
	}

	new->next = NULL;

	if(!w) new->id = 1; /* bah c'est le 1er */
	else new->id = w->id + 1; /* on incrmente l'id */

	parv2msgn(parc, parv, 2, new->msg, 250);
	new->level = level;

	if(!w) welcomehead = new; /* ben c'est le 1er */
	else w->next = new;

	csntc(nick, "Welcome bien ajout au level %d", level);
	csntc(nick, "\2[Message de bienvenue #%d]\2 - %s", new->id, new->msg);

	write_welcome();
	return 1;
}

int del_welcome(aNick *nick, aChan *c, int parc, char **parv)
{
	struct welcomeinfo *current = NULL, *previous = NULL;
	int id;

	if(parc < 1) return 0;
	if(!is_num(parv[1])) return csntc(nick, "L'id doit tre un nombre.");

	id = atoi(parv[1]);
	current = welcomehead;

	while(current)
	{
		if(id == current->id) break;
		previous = current;
		current = current->next;
	}
	if(!current)
		return csntc(nick, "Aucune message ayant pour id %d", id);

	if(!previous) welcomehead = current->next; /* ben c'etait le 1er */
	else previous->next = current->next;
	free(current);
	current = welcomehead;

	id = 1;
	while(current)
	{
		current->id = id;
		current = current->next;
		id++;
	}
	csntc(nick, "Le message ayant pour id %s a bien t supprim (un total de %d messages dsormais)", parv[1], id - 1);
	write_welcome();
	return 1;
}

int list_welcome(aNick *nick, aChan *c, int parc, char **parv)
{
	struct welcomeinfo *w;

	if(!welcomehead) return csntc(nick, "Aucun message de bienvenue.");

	csntc(nick, "\2#Id Level Message");
	for(w=welcomehead;w;w=w->next) csntc(nick, "#%d %d %s", w->id, w->level, w->msg);

	return 1;
}

void send_welcome(const char *num, struct welcomeinfo *w)
{
	putserv("%s %s %s :\2[Message de bienvenue #%d]\2 - %s\n", ws.num, GetConf(CF_PRIVWELCOME) ? TOKEN_PRIVMSG: TOKEN_NOTICE, num, w->id, w->msg);
	return;
}

#undef RAND_MAX
#define RAND_MAX 99999999
void choose_welcome(const char *num)
{
	int rnd;
	struct welcomeinfo *w;

	if(!welcomehead) return;
	rnd = 1;
	while(rnd != 0)
	{
		rnd = rand()%100;
		for(w=welcomehead;w;w=w->next) if(w->level > rnd && rnd%3) break;
		if(w)
		{
			send_welcome(num, w);
			return;
		}
		else rnd--;
	}
	w = welcomehead;
	send_welcome(num, w);
	return;
}

void write_welcome()
{
	FILE *fp;
	struct welcomeinfo *w;

	if(!(fp = fopen(WELCOME_FILE, "w"))) return;
	fprintf(fp, "VERSION %d\n", USRS_VERSION);
	fprintf(fp, "%d\n%s\n%s\n", GetConf(CF_WELCOME), user_motd, admin_motd);
	for(w = welcomehead;w;w=w->next) fprintf(fp, "%d :%s\n", w->level, w->msg);
   	fclose(fp);
   	return;
}

void add_loadwelcome(int level, char *msg)
{
	struct welcomeinfo *w, *new;

	new = (struct welcomeinfo *) calloc(1, sizeof(struct welcomeinfo));
   	if(!new)
   	{
	   	putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour welcomeinfo !\n");
	   	return;
   	}

	w = welcomehead;
	while(w)
	{
		if(!w->next) break;
		else w = w->next;
	}

	new->next = NULL;

	if(!w) new->id = 1; /* bah c'est le 1er */
	else new->id = w->id + 1; /* on incrmente l'id */

	strncpy(new->msg, msg, 250);
	new->level = level;

	if(!w) welcomehead = new; /* ben c'est le 1er */
	else w->next = new;

	return;
}

void load_welcome()
{
	char buff[420], *tmp, msg[250];
	int a=0, t=0, fileversion = 1, g;
	FILE *fp;

	if(!(fp = fopen(WELCOME_FILE, "r"))) return;
	while(fgets(buff, 420, fp))
   	{
		strip_newline(buff);
		if(!strncmp(buff, "VERSION", 7))
		{
			if((tmp=strchr(buff, ' '))) tmp++;
			fileversion = atoi(tmp);
		}
		else if(t==1 && fileversion>=8) sscanf(buff, "%d",&g);
		else if(t==2 && fileversion>=8) strncpy(user_motd, buff, 390);
		else if(t==3 && fileversion>=8) strncpy(admin_motd, buff, 390);
		else
		{
			tmp = strchr(buff, ':');
	  		if (*tmp == ':') tmp++;
			strncpy(msg, tmp, 250);
			a = atoi(strtok(buff, " "));
			add_loadwelcome(a, msg);
		}
		t++;
	}
	fclose(fp);
	if(fileversion<8)
	{
		if(!(fp = fopen(SETWELCOME_FILE, "r"))) return;
		if(fgets(buff, 420, fp)) sscanf(buff, "%d", &g);
		fclose(fp);
	}
	if(g) ConfFlag |= CF_WELCOME;
	return;
}
#endif
